def supports_color() -> bool: ...

class Style:
    def DEBUG(self, text: str) -> str: ...
    def INFO(self, text: str) -> str: ...
    def SUCCESS(self, text: str) -> str: ...
    def WARNING(self, text: str) -> str: ...
    def ERROR(self, text: str) -> str: ...

def make_style(config_string: str = ...) -> Style: ...
def no_style() -> Style: ...
def color_style() -> Style: ...
